@@ -14,6 +14,7 @@ logger = logging.Logger(__name__)
 TEAMJAM_SUMMARY_COLUMNS = [
     "Calloff", "Injury", "JamScore", "Lead",
     "Lost", "NoInitial", "StarPass", "TotalScore", "jammer_name", "jammer_number",
+    "pivot_name", "pivot_number",
     "Skaters"]
 
 
@@ -37,6 +38,8 @@ def load_json_derby_game(game_json) -> DerbyGame:
     pdf_roster = extract_roster(pdf_game_state,
                                 game_data_dict["team_1"],
                                 game_data_dict["team_2"])
+    pdf_ref_roster = extract_officials_roster(pdf_game_state, "Ref")
+    pdf_nso_roster = extract_officials_roster(pdf_game_state, "Nso")
     logger.debug("Extracting jam data")
     pdf_game_data = extract_jam_data(pdf_game_state, pdf_roster)
     logger.debug("Extracting penalties")
@@ -46,7 +49,8 @@ def load_json_derby_game(game_json) -> DerbyGame:
         pdf_team_colors = extract_team_colors(pdf_game_state)
     except Exception as e:
         logger.warn(f"Failed to extract team colors. Exception: {e}")
-    return DerbyGame(pdf_game_data, game_data_dict, pdf_penalties, pdf_team_colors)
+    return DerbyGame(pdf_game_data, game_data_dict, pdf_penalties, pdf_team_colors,
+                     pdf_roster, pdf_ref_roster, pdf_nso_roster)
 
 
 def json_to_game_dataframe(game_json: Dict[Any, Any]) -> pd.DataFrame:
@@ -276,7 +280,11 @@ def build_jam_dataframe(pdf_game_state: pd.DataFrame) -> pd.DataFrame:
     
     pdf_jam_data = pdf_period[
         pdf_period.keychunk_2.str.startswith("Jam(")].copy()
-    # All the "Jam" fields have at least 3 chunks
+
+    # Make sure all the "Jam" fields have at least 4 chunks. Defensive coding
+    # against a bug erevrav reported 20230311
+    pdf_jam_data = pdf_jam_data[[len(x) >= 4 for x in pdf_jam_data.key_chunks]]
+
     pdf_jam_data["keychunk_3"] = [x[3] for x in pdf_jam_data.key_chunks]
 
     logger.debug(f"Found {len(pdf_jam_data)} Jam rows.")
@@ -408,6 +416,10 @@ def extract_roster(pdf_game_state: pd.DataFrame,
         ["Id", "Name", "RosterNumber", "team"]
     )]
     pdf_roster = pdf_game_state_roster.pivot(index="skater", columns="roster_key", values="value")
+    logger.debug("pdf_roster columns: " + str(pdf_roster.columns) + 
+                 ". Before dropping nulls, length: " + str(len(pdf_roster)))
+    pdf_roster = pdf_roster[pdf_roster.Id.notnull()]
+    logger.debug("After dropping nulls, length: " + str(len(pdf_roster)))
 
     skaterid_team_map = dict(zip(*[pdf_game_state_roster["skater"], pdf_game_state_roster["team"]]))
     pdf_roster["team"] = [skaterid_team_map[skater] for skater in pdf_roster.Id]
@@ -465,9 +477,19 @@ def process_team_jam_info(pdf_game_state: pd.DataFrame, team_number: int,
     pdf_ateamjams_summary = pdf_ateamjams_summary.merge(pdf_roster.rename(
         columns={"Id": "jammer_id", "Name": "jammer_name", "RosterNumber": "jammer_number"}),
         on="jammer_id", how="left")
-
     logger.debug(f"After adding jammer info: {len(pdf_ateamjams_summary)}")
 
+    # add pivot info
+    pdf_ateamjams_pivots = pdf_ateamjams_data[
+        pdf_ateamjams_data.key.str.endswith("Fielding(Pivot).Skater")][
+            ["prd_jam", "value"]].rename(columns={"value": "pivot_id"})
+    pdf_ateamjams_summary = pdf_ateamjams_summary.merge(pdf_ateamjams_pivots,
+        on="prd_jam", how="left")
+    pdf_ateamjams_summary = pdf_ateamjams_summary.merge(pdf_roster.rename(
+        columns={"Id": "pivot_id", "Name": "pivot_name", "RosterNumber": "pivot_number"}),
+        on="pivot_id", how="left")
+    logger.debug(f"After adding pivot info: {len(pdf_ateamjams_summary)}")
+
     # add list of skater names per jam
     pdf_jam_skater_lists = extract_team_perjam_skaters(pdf_ateamjams_data, pdf_roster)
     pdf_ateamjams_summary = pdf_ateamjams_summary.merge(
@@ -525,10 +547,9 @@ def extract_team_perjam_skaters(pdf_ateamjams_data: pd.DataFrame,
     if len(pdf_ateamjams_data_skaters_withname) == 0:
         pdf_ateamjams_data_skaters_withname = pdf_ateamjams_data_skaters.merge(
             pdf_roster, on="Id", how="left")
-        logger.warn("Failed to join skater data with roster. Merged with left join, "
+        logger.debug("Failed to join skater data with roster. Merged with left join, "
                     f"rows {len(pdf_ateamjams_data_skaters_withname)}")
     logger.debug(f"    After roster merge, team jam skater data: {len(pdf_ateamjams_data_skaters_withname)}")
-
     pdf_jam_skater_lists = pdf_ateamjams_data_skaters_withname.groupby(
         "prd_jam")["Name"].apply(list).reset_index()
     pdf_jam_skater_lists = pdf_jam_skater_lists.rename(columns={"Name": "Skaters"})
@@ -598,20 +619,64 @@ def extract_penalties(pdf_game_state: pd.DataFrame,
     logger.debug("extract_penalties begin")
     pdf_penalty_gamedata = pdf_game_state[(pdf_game_state.n_key_chunks == 5)].copy()
     pdf_penalty_gamedata["keychunk_2"] = [chunks[2] for chunks in pdf_penalty_gamedata.key_chunks]
-    pdf_penalty_gamedata["keychunk_3"] = [chunks[3] for chunks in pdf_penalty_gamedata.key_chunks]
+
+    # this value includes the string "Penalty(" and trailing ")"
+    pdf_penalty_gamedata["penalty_number"] = [chunks[3] for chunks in pdf_penalty_gamedata.key_chunks]
+
+    # Ignore "Penalty(0)". Those aren't regular penalties. They seem to get generated when
+    # someone fouls out.
     pdf_penalty_gamedata = pdf_penalty_gamedata[
-        pdf_penalty_gamedata.keychunk_3.str.startswith("Penalty(")]
+        pdf_penalty_gamedata.penalty_number.str.startswith("Penalty(") &
+        ~pdf_penalty_gamedata.penalty_number.str.startswith("Penalty(0)")]
+
     logger.debug(f"    Rows with `Penalty(`: {len(pdf_penalty_gamedata)}")
-    pdf_penalty_gamedata["penalty_key"] = [chunks[4] for chunks in pdf_penalty_gamedata.key_chunks]
-    pdf_penalty_gamedata["Id"] = [chunk[len("Skater("):-1]
-                                  for chunk in pdf_penalty_gamedata.keychunk_2]
-    pdf_penalty_gamedata = pdf_penalty_gamedata.merge(
-        pdf_roster[["Id", "Name", "team"]], on="Id")
-    logger.debug(f"    After merging with roster: {len(pdf_penalty_gamedata)}")
-    pdf_penalties = pdf_penalty_gamedata[pdf_penalty_gamedata.penalty_key == "Code"]
-    pdf_penalties = pdf_penalties.rename(columns={"value": "penalty_code"})
-    pdf_penalties = pdf_penalties[["Name", "team", "penalty_code"]]
-    logger.debug(f"    Before merge with penalty codes: {len(pdf_penalties)}")
+    pdf_penalty_gamedata["penalty_variable"] = [chunks[4] for chunks in pdf_penalty_gamedata.key_chunks]
+    pdf_penalty_gamedata["SkaterId"] = [chunk[len("Skater("):-1]
+                                for chunk in pdf_penalty_gamedata.keychunk_2]
+
+    # make a unique identifier for this penalty from the combination of skater id and penalty number.
+    # Cast skaterid to string because if it's empty it gets float.
+    # Cast penalty number to string just in case.
+    pdf_penalty_gamedata["penalty_id"] = pdf_penalty_gamedata["SkaterId"].astype(str) + "___" + pdf_penalty_gamedata["penalty_number"].astype(str)
+
+    # Drop extraneous columns
+    pdf_penalties_long = pdf_penalty_gamedata[["penalty_id", "penalty_variable", "value"]]
+
+    # Pivot to create different columns for each different penalty key
+    pdf_penalties = pdf_penalties_long.pivot(index="penalty_id", 
+                                            columns="penalty_variable",
+                                            values="value")
+    # restore skater ID as "Id"
+    pdf_penalties["Id"] = [x.split("___")[0] for x in pdf_penalties.index]
+
+    # dummy up columns if they're missing, which happens if no penalties yet
+    for col in ["PeriodNumber", "JamNumber", "Code", "Served", "Serving", "Time"]:
+        if col not in pdf_penalties:
+            pdf_penalties[col] = None
+
+    # "Served" and "Serving" are a bit silly to separate. Combine them
+    pdf_penalties["Status"] = [
+        "Not Yet" if not served and not serving
+        else "Serving" if served and serving
+        else "Served" if served and not serving
+        else "Unknown"
+        for served, serving in zip(*[pdf_penalties.Served, pdf_penalties.Serving])
+    ]
+    pdf_penalties = pdf_penalties.drop(columns=["Served", "Serving"])
+
+    # Drop extraneous columns
+    pdf_penalties = pdf_penalties[["Id", "PeriodNumber", "JamNumber",
+                                   "Code", "Status", "Time"]]
+    pdf_penalties["prd_jam"] = [
+        f"{period}:{'0' if (jam < 10) else ''}{jam}" 
+        for period, jam in zip(*[pdf_penalties.PeriodNumber, pdf_penalties.JamNumber])]
+    # cast is necessary if there are no penalties yet
+    pdf_penalties = pdf_penalties.astype({"prd_jam": str})
+    pdf_penalties = pdf_penalties.rename(columns={"Code": "penalty_code"})
+
+    logger.debug(f"    Before merging with roster: {len(pdf_penalties)}")
+    pdf_penalties = pdf_penalties.merge(pdf_roster[["Id", "Name", "RosterNumber", "team"]], on="Id")
+    logger.debug(f"    After merging with roster: {len(pdf_penalties)}")
 
     # add penalty names
     pdf_penalty_codes_names = build_penalty_code_name_map(pdf_game_state, json_major_version)
@@ -703,5 +768,43 @@ def extract_team_colors(pdf_game_data:pd.DataFrame) -> pd.DataFrame:
     if succeeded:
         logger.debug(f"Team colors: {pdf_team_colors}")
     else:
-        logger.info("Could not load team colors. Team colors will be missing from plots.")
-    return pdf_team_colors
\ No newline at end of file
+        logger.debug("Could not load team colors. Team colors will be missing from plots.")
+    return pdf_team_colors
+
+
+def extract_officials_roster(pdf_game_state: pd.DataFrame,
+                             Ref_or_Nso: str) -> pd.DataFrame:
+    """Extract the officials roster from the game data.
+    Ref_or_Nso should be "Ref" or "Nso"
+    """
+    try:
+        pdf_game_state_officials_roster = pdf_game_state.loc[
+            pdf_game_state.key.str.contains(
+                f".{Ref_or_Nso}\(")
+        ].copy()
+
+        pdf_game_state_officials_roster["roster_key"] = [
+            chunks[2] for chunks in pdf_game_state_officials_roster.key_chunks]
+        # dump a bunch of extraneous columns
+        pdf_game_state_officials_roster = pdf_game_state_officials_roster[
+            pdf_game_state_officials_roster.roster_key.isin(
+            ["Name", "Role"]
+        )]
+        pdf_officials_roster = pdf_game_state_officials_roster.pivot(
+            index="keychunk_1", columns="roster_key", values="value")
+        pdf_officials_roster = pdf_officials_roster.sort_values("Name")
+        # this is dumb but necessary
+        pdf_officials_roster = pd.DataFrame({
+            "Name": list(pdf_officials_roster.Name),
+            "Role": list(pdf_officials_roster.Role),
+        })
+    except Exception as e:
+        logger.debug(f"Could not load officials roster: {e}")
+        pdf_officials_roster = pd.DataFrame({
+            "Name": [],
+            "Role": []
+        })
+    return pdf_officials_roster
+
+
+    
\ No newline at end of file
